/* Color.h declares class Color.
 * ...
 * Class Invariant:
 *      myColorValue == RED || myColorValue == ORANGE ||
 *      myColorValue == YELLOW || myColorValue == GREEN ||
 *      myColorValue == BLUE || myColorValue == INDIGO ||
 *      myColorValue ==  VIOLET.
 ****************************************************************/
 
#ifndef COLOR
#define COLOR

#include <iostream>                           // istream, ostream
using namespace std;

enum ColorValue {COLOR_UNDERFLOW = -1, 
                 RED, ORANGE, YELLOW, GREEN,
                 BLUE, INDIGO, VIOLET,
                 COLOR_OVERFLOW,
                 NUMBER_OF_COLORS = 7};
class Color
{
 public:
   Color();                                            // constructors
   Color(ColorValue initialColorValue);

   double wavelength() const;                          // properties
   double frequency() const;

   void read(istream& in);                             // I/O
   void print(ostream& out) const;
                                                       // relationals
   friend bool operator== (const Color& left, const Color& right);
   friend bool operator!= (const Color& left, const Color& right);
   friend bool operator<  (const Color& left, const Color& right);
   friend bool operator>  (const Color& left, const Color& right);
   friend bool operator<= (const Color& left, const Color& right);
   friend bool operator>= (const Color& left, const Color& right);

   Color operator++();                                 // prefix ++
   Color operator++(int);                              // postfix ++

   Color operator--();                                 // prefix --
   Color operator--(int);                              // postfix --


 private:
   ColorValue myColorValue;
};

// ------ Default-value constructor ----------------
inline Color::Color()
{
   myColorValue = RED;
}

// ------- Explicit-value Constructor -----------------------------
#include <cassert>                                   // assert()
using namespace std;

inline Color::Color(ColorValue initialColorValue)
{
   assert(RED <= initialColorValue && initialColorValue <= VIOLET);
   myColorValue = initialColorValue;
}

// -------- Function Input --------------------------------------
inline istream& operator>>(istream& in, Color& theColor)
{
   theColor.read(in);
   return in;
}

// -------- Function Output --------------------------------------
inline ostream& operator<<(ostream& out, const Color& theColor)
{
   theColor.print(out);
   return out;
}

// -------- Equality ---------------------------------------------
inline bool operator==(const Color & left, const Color & right)
{
   return left.myColorValue == right.myColorValue;
}

// -------- Less-than --------------------------------------------
inline bool operator<(const Color & left, const Color & right)
{
   return left.myColorValue < right.myColorValue;
}

// -------- Less-than-or-equal -----------------------------------
inline bool operator<=(const Color & left, const Color & right)
{
   return left.myColorValue <= right.myColorValue;
}


#endif
